@@ -5,14 +5,15 @@
 package DateTime::Set;
 
 use strict;
-
+use Carp;
 use Params::Validate qw( validate SCALAR BOOLEAN OBJECT CODEREF ARRAYREF );
-use Set::Infinite '0.45';
+use DateTime::Span;
+use Set::Infinite '0.44_04';
 $Set::Infinite::PRETTY_PRINT = 1;   # enable Set::Infinite debug
 
 use vars qw( @ISA $VERSION );
 
-$VERSION = '0.00_18';
+$VERSION = '0.00_20';
 
 use constant INFINITY     =>       100 ** 100 ** 100 ;
 use constant NEG_INFINITY => -1 * (100 ** 100 ** 100);
@@ -36,18 +37,22 @@ sub _add_callback {
     }
     my $result = $set->new( $min );
     return $result;
-}; 
+}
 
-sub add {
-    my ($self, %parm) = @_;
-    my $dur;
-    if (exists $parm{duration}) {
-        $dur = $parm{duration}->clone;
-    }
-    else {
-        $dur = new DateTime::Duration( %parm );
-    }
-    my $result = $self->{set}->iterate( \&_add_callback, $dur );
+sub add { shift->add_duration( DateTime::Duration->new(@_) ) }
+
+sub subtract { return shift->subtract_duration( DateTime::Duration->new(@_) ) }
+
+sub subtract_duration { return $_[0]->add_duration( $_[1]->inverse ) }
+
+sub add_duration {
+    my ( $self, $dur ) = @_;
+
+    # $dur should be cloned because if the set is a
+    # recurrence then the callback will not be used
+    # immediately - then $dur must be "immutable".
+
+    my $result = $self->{set}->iterate( \&_add_callback, $dur->clone );
 
     ### this code would enable 'subroutine method' behaviour
     # $self->{set} = $result;
@@ -63,6 +68,8 @@ sub add {
 # the set elements become immutable
 sub new {
     my $class = shift;
+    carp "new( \%param ) is deprecated. Use from_recurrence() / from_datetimes() instead,"
+        if @_;
     my %args = validate( @_,
                          { start =>
                            { type => OBJECT,
@@ -72,7 +79,15 @@ sub new {
                            { type => OBJECT,
                              optional => 1,
                            },
-                           recurrence =>
+                           recurrence =>      # "next" alias 
+                           { type => CODEREF,
+                             optional => 1,
+                           },
+                           next =>
+                           { type => CODEREF,
+                             optional => 1,
+                           },
+                           previous =>
                            { type => CODEREF,
                              optional => 1,
                            },
@@ -83,6 +98,9 @@ sub new {
                          }
                        );
     my $self = {};
+
+    $args{next} = $args{recurrence} if exists $args{recurrence};
+
     if (exists $args{dates}) {
         $self->{set} = Set::Infinite->new;
         # warn "new: inserting @{ $args{dates} }";
@@ -91,22 +109,81 @@ sub new {
             $self->{set} = $self->{set}->union( $_->clone );
         }
     }
-    elsif (exists $args{recurrence}) {
+    elsif (exists $args{next}) {
         # Set::Infinity->iterate() builds a "set-function" with a callback:
         my $start = ( exists $args{start} ) ? $args{start} : NEG_INFINITY;
         my $end =   ( exists $args{end} )   ? $args{end}   : INFINITY;
         $start = $start->clone if ref($start);
         $end =   $end->clone   if ref($end);
         my $tmp_set = Set::Infinite->new( $start, $end );
-        $self->{set} = _recurrence_callback( $tmp_set, $args{recurrence} );  
+        $self->{set} = _recurrence_callback( $tmp_set, $args{next} );  
+    }
+    elsif (exists $args{previous}) {
+        die '"previous =>" argument not implemented';
     }
     else {
+        # no arguments => return an empty set (or should die?)
         $self->{set} = Set::Infinite->new;
     }
     bless $self, $class;
     return $self;
 }
 
+sub from_recurrence {
+    my $class = shift;
+    # note: not using validate() because this is too complex...
+    my %args = @_;
+    my %param;
+    # Parameter renaming, such that we can use either
+    #   recurrence => xxx   or   next => xxx, previous => xxx
+    $param{next} = delete $args{recurrence}  or
+    $param{next} = delete $args{next};
+    $param{previous} = $args{previous}  and  delete $args{previous};
+    $param{span} = $args{span}  and  delete $args{span};
+    # they might be specifying a span using begin / end
+    $param{span} = new DateTime::Span( %args ) if keys %args;
+    # otherwise, it is unbounded
+    $param{span}->{set} = Set::Infinite->new( NEG_INFINITY, INFINITY )
+        unless exists $param{span}->{set};
+    my $self = {};
+    if (exists $param{next}) {
+        $self->{set} = _recurrence_callback( $param{span}->{set}, $param{next} );  
+        bless $self, $class;
+    }
+    elsif (exists $param{previous}) {
+        die '"previous =>" argument not implemented in from_recurrence()';
+    }
+    else {
+        die "Not enough arguments in from_recurrence()";
+    }
+    return $self;
+}
+
+sub from_datetimes {
+    my $class = shift;
+    my %args = validate( @_,
+                         { dates => 
+                           { type => ARRAYREF,
+                           },
+                         }
+                       );
+    my $self = {};
+    $self->{set} = Set::Infinite->new;
+    # possible optimization: sort dates and use "push"
+    for( @{ $args{dates} } ) {
+        $self->{set} = $self->{set}->union( $_->clone );
+    }
+
+    bless $self, $class;
+    return $self;
+}
+
+sub empty_set {
+    my $class = shift;
+
+    return bless { set => Set::Infinite->new }, $class;
+}
+
 sub clone { 
     bless { 
         set => $_[0]->{set}->copy,
@@ -120,21 +197,21 @@ sub clone {
 # These recurrences are simple lists of dates.
 #
 # this is an internal callback - it is not an object method!
-# Used by: new( recurrence => )
+# Used by: new( next => )
 #
 # The recurrence generation is based on an idea from Dave Rolsky.
 #
 sub _recurrence_callback {
     # warn "_recurrence args: @_";
     # note: $_[0] is a Set::Infinite object
-    my ( $set, $callback ) = @_;    
+    my ( $set, $callback, $callback_info ) = @_;    
 
     # test for the special case when we have an infinite recurrence
 
     if ($set->min == NEG_INFINITY ||
         $set->max == INFINITY) {
 
-        return _setup_infinite_recurrence( $set, $callback );
+        return _setup_infinite_recurrence( $set, $callback, $callback_info );
     }
     else {
 
@@ -143,17 +220,29 @@ sub _recurrence_callback {
 }
 
 sub _setup_infinite_recurrence {
-    my ( $set, $callback ) = @_;
+    my ( $set, $callback, $callback_info ) = @_;
 
     # warn "_recurrence called with inf argument";
-    return NEG_INFINITY if $set->min == NEG_INFINITY && $set->max == NEG_INFINITY;
-    return INFINITY if $set->min == INFINITY && $set->max == INFINITY;
+
+    # these should never happen, but we have to test anyway:
+    return $set->new( NEG_INFINITY ) 
+        if $set->min == NEG_INFINITY && $set->max == NEG_INFINITY;
+    return $set->new( INFINITY ) 
+        if $set->min == INFINITY && $set->max == INFINITY;
+
+    # set up a hash to store additional info on the callback,
+    # such as direction (next/previous) and the 
+    # approximate time between events
+    $callback_info = { 
+        freq => undef,
+    } unless defined $callback_info;
+
     # return an internal "_function", such that we can 
     # backtrack and solve the equation later.
     $set = $set->copy;
     my $func = $set->_function( 'iterate', 
         sub {
-            _recurrence_callback( $_[0], $callback );
+            _recurrence_callback( $_[0], $callback, $callback_info );
         }
     );
 
@@ -180,7 +269,7 @@ sub _setup_infinite_recurrence {
             $set->new( $min->clone ), 
             $next_set->_function( 'iterate',
                 sub {
-                    _recurrence_callback( $_[0], $callback );
+                    _recurrence_callback( $_[0], $callback, $callback_info );
                 } ) );
         # warn "RECURR: preparing first: $min ; $next; got @first";
         $func->{first} = \@first;
@@ -194,9 +283,9 @@ sub _setup_infinite_recurrence {
     else {
         my $max = $func->max;
         # iterate to find previous value
-        my $previous = _callback_previous( $max, $callback );
+        my $previous = _callback_previous( $max, $callback, $callback_info );
         # warn "previous: ".$previous->ymd;
-        my $previous2 = _callback_previous( $previous, $callback );
+        my $previous2 = _callback_previous( $previous, $callback, $callback_info );
         # my $previous3 = _callback_previous( $previous2, $callback );
         # warn "RECURR: preparing last: ".$previous2->ymd." ; ".$previous3->ymd;
         my $previous_set = $set->intersection( NEG_INFINITY, $previous2->clone );
@@ -205,7 +294,7 @@ sub _setup_infinite_recurrence {
             $set->new( $max->clone ),
             $previous_set->_function( 'iterate',
                 sub {
-                    _recurrence_callback( $_[0], $callback );
+                    _recurrence_callback( $_[0], $callback, $callback_info );
                 } ) );
         # warn "RECURR: preparing last: $max ; $previous; got @last";
         $func->{last} = \@last;
@@ -243,16 +332,43 @@ sub _setup_finite_recurrence {
 
 # returns the "previous" value in a callback recurrence
 sub _callback_previous {
-    my ($value, $callback) = @_; 
+    my ($value, $callback, $callback_info) = @_; 
     my $previous = $value->clone;
     # go back at least an year...
     # TODO: memoize.
     # TODO: binary search to find out what's the best subtract() unit.
-    $previous->subtract( months => 13 );  
+
+    my $freq = ${$callback_info}{freq};
+    unless (defined $freq) { 
+
+        # This is called just once, to setup the recurrence frequency
+        # The use of 'next' to simulate 'previous' might be
+        # considered a hack. 
+        # The program will warn() if it this is not working properly.
+
+        my $next = $value->clone;
+        $next = &$callback( $next );
+        $freq = $next - $previous;
+        my %freq = $freq->deltas;
+        $freq{$_} = -int( $freq{$_} * 2 ) for keys %freq; 
+        $freq = new DateTime::Duration( %freq );
+
+        # save it for future use with this same recurrence
+        ${$callback_info}{freq} = $freq;
+
+        # my @freq = $freq->deltas;
+        # warn "freq is @freq";
+    }
+
+    $previous->add_duration( $freq );  
+    # warn "callback is $callback";
     # warn "current is ".$value->ymd." previous is ".$previous->ymd;
     $previous = &$callback( $previous );
+    # warn " previous got ".$previous->ymd;
     if ($previous >= $value) {
-        die "_callback_previous iterator can't find a previous value, got ".$previous->ymd." before ".$value->ymd;
+        # This error might happen if the event frequency oscilates widely
+        # (more than 100% of difference from one interval to next)
+        warn "_callback_previous iterator can't find a previous value, got ".$previous->ymd." before ".$value->ymd;
     }
     my $previous1;
     while (1) {
@@ -288,13 +404,28 @@ sub previous {
     return $head;
 }
 
+sub as_list {
+    my ($self) = shift;
+    return undef unless ref( $self->{set} );
+    return undef if $self->{set}->is_too_complex;  # undef = no begin/end
+    return if $self->{set}->is_null;  # nothing = empty
+    my @result;
+    # we should extract _copies_ of the set elements,
+    # such that the user can't modify the set indirectly
+    for ( @{ $self->{set}->{list}  } ) {
+        push @result, $_->{a}->clone
+            if ref( $_->{a} );   # we don't want to return INFINITY value
+    }
+    return @result;
+}
+
 # Set::Infinite methods
 
 sub intersection {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
     my $tmp = $class->new();
-    $set2 = $class->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
+    $set2 = $class->from_datetimes( dates => [ $set2 ] ) unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->intersection( $set2->{set} );
     return $tmp;
 }
@@ -303,7 +434,7 @@ sub intersects {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
     my $tmp = $class->new();
-    $set2 = $class->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
+    $set2 = $class->from_datetimes( dates => [ $set2 ] ) unless $set2->can( 'union' );
     return $set1->{set}->intersects( $set2->{set} );
 }
 
@@ -311,7 +442,7 @@ sub contains {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
     my $tmp = $class->new();
-    $set2 = $class->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
+    $set2 = $class->from_datetimes( dates => [ $set2 ] ) unless $set2->can( 'union' );
     return $set1->{set}->contains( $set2->{set} );
 }
 
@@ -319,8 +450,10 @@ sub union {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
     my $tmp = $class->new();
-    $set2 = $class->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
+    $set2 = $class->from_datetimes( dates => [ $set2 ] ) unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->union( $set2->{set} );
+    bless $tmp, 'DateTime::SpanSet' 
+        if $set2->isa('DateTime::Span') or $set2->isa('DateTime::SpanSet');
     return $tmp;
 }
 
@@ -329,7 +462,7 @@ sub complement {
     my $class = ref($set1);
     my $tmp = $class->new();
     if (defined $set2) {
-        $set2 = $class->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
+        $set2 = $class->from_datetimes( dates => [ $set2 ] ) unless $set2->can( 'union' );
         $tmp->{set} = $set1->{set}->complement( $set2->{set} );
     }
     else {
@@ -375,20 +508,19 @@ DateTime::Set - Date/time sets math
     use DateTime::Set;
 
     $date1 = DateTime->new( year => 2002, month => 3, day => 11 );
-    $set1 = DateTime::Set->new( dates => [ $date1 ] );
+    $set1 = DateTime::Set->from_datetimes( dates => [ $date1 ] );
     #  set1 = 2002-03-11
 
     $date2 = DateTime->new( year => 2003, month => 4, day => 12 );
-    $set2 = DateTime::Set->new( dates => [ $date1, $date2 ] );
+    $set2 = DateTime::Set->from_datetimes( dates => [ $date1, $date2 ] );
     #  set2 = 2002-03-11, and 2003-04-12
 
     # a 'monthly' recurrence:
-    $set = DateTime::Set->new( 
+    $set = DateTime::Set->from_recurrence( 
         recurrence => sub {
             $_[0]->truncate( to => 'month' )->add( months => 1 )
         },
-        start => $date1,    # optional
-        end => $date2,      # optional
+        span => $date_span1,    # optional span
     );
 
     $set = $set1->union( $set2 );         # like "OR", "insert", "both"
@@ -429,52 +561,75 @@ Wednesday between 2003-03-05 and 2004-01-07".
 
 =over 4
 
-=item * new
+=item * from_datetimes
 
-Creates a new set.  The set can either be a list of dates, or it can
-be specified via a "recurrence" callback.
+Creates a new set from a list of dates.
 
-To create a set from a list of dates:
+   $dates = DateTime::Set->from_datetimes( dates => [ $dt1, $dt2, $dt3 ] );
 
-   $dates = DateTime::Set->new( dates => [ $dt1, $dt2, $dt3 ] );
+=item * from_recurrence
 
-To create a set as a recurrence:
+Creates a new set specified via a "recurrence" callback.
 
-    $months = DateTime::Set->new( 
-        start => $today, 
-        end => $today_plus_one_year,
-        recurrence => sub { $_[0]->truncate( to => 'month' )->add( months => 1 ) }, 
+    $months = DateTime::Set->from_recurrence( 
+        span => $dt_span_this_year,    # optional span
+        recurrence => sub { 
+            $_[0]->truncate( to => 'month' )->add( months => 1 ) 
+        }, 
     );
 
-The "start" and "end" parameters are both optional.  If no "start"
-parameter is given then the set is assumed to start at negative
-infinity.  Similarly, if no "end" parameter is given then the set is
-assumed to end at infinity.
+The C<span> parameter is optional. It must be a C<DateTime::Span> object.
 
-=item * add
+The span can also be specified using 
+C<begin> / C<after> and C<end> / C<before> DateTime objects, 
+as in the C<DateTime::Span> constructor. 
+In this case, if there is a C<span> parameter it will be ignored.
 
-    $new_set = $set->add( year => 1 );
+    $months = DateTime::Set->from_recurrence(
+        after => $dt_now,
+        recurrence => sub {
+            $_[0]->truncate( to => 'month' )->add( months => 1 )
+        },
+    );
+
+=item * empty_set
+
+Creates a new empty set.
+
+=item * add_duration( $duration )
 
     $dtd = new DateTime::Duration( year => 1 );
     $new_set = $set->add( duration => $dtd );
 
 This method returns a new set which is the same as the existing set
-plus the specified duration.
+with the specified duration added to every element of the set.
+
+=item * add
 
     $meetings_2004 = $meetings_2003->add( years => 1 );
 
-This method takes the same parameters as allowed by
-C<DateTime->add()>.  It can also take a "duration" parameter, which
-should be a C<DateTime::Duration> object.  If this parameter is given
-then all others are ignored.
+This method creates a new C<DateTime::Duration> object based on the
+parameters given and passes it to the C<add_duration()> method.
+
+=item * subtract_duration( $duration_object )
+
+When given a C<DateTime::Duration> object, this method simply calls
+C<invert()> on that object and passes that new duration to the
+C<add_duration> method.
+
+=item * subtract( DateTime::Duration->new parameters )
+
+Like C<add()>, this is syntactic sugar for the C<subtract_duration()>
+method.
 
 =item * min / max
 
-First or last dates in the set.
+The first and last dates in the set.  These methods may return
+C<undef> if the set is empty.
 
 =item * span
 
-The total span of the set, as a DateTime::Span.
+Returns the total span of the set, as a C<DateTime::Span> object.
 
 =item * iterator / next
 
@@ -485,22 +640,37 @@ These methods can be used to iterate over the dates in a set.
         print $dt->ymd;
     }
 
-The C<next()> or C<previous()> return C<undef> when there are no more
-datetimes in the iterator.
+The C<next()> or C<previous()> method will return C<undef> when there
+are no more datetimes in the iterator.
 
 Obviously, if a set is specified as a recurrence and has no fixed end
 datetime, then it may never stop returning datetimes.  User beware!
 
-=item union / intersection / complement
+=item * as_list
 
-These set operations result in a DateTime::Set.
+Returns a list of C<DateTime> objects.
+
+If a set is specified as a recurrence and has no fixed begin or end
+datetimes, then C<as_list> will return C<undef>.  Please note that
+this is explicitly not an empty list, since an empty list is a valid
+return value for empty sets!
+
+=item * union / intersection / complement
+
+Set operations.
 
     $set = $set1->union( $set2 );         # like "OR", "insert", "both"
     $set = $set1->complement( $set2 );    # like "delete", "remove"
     $set = $set1->intersection( $set2 );  # like "AND", "while"
     $set = $set1->complement;             # like "NOT", "negate", "invert"
 
-=item intersects / contains
+The C<union> 
+with a C<DateTime::Span> or a C<DateTime::SpanSet> object
+returns a C<DateTime::SpanSet> object.
+
+All other operations always return a C<DateTime::Set>.
+
+=item * intersects / contains
 
 These set operations result in a boolean value.
 
